@@ -7,6 +7,7 @@ use base qw|Jifty::DBI::Filter|;
 use DateTime                  ();
 use DateTime::Format::ISO8601 ();
 use DateTime::Format::Strptime ();
+use Carp ();
 
 use constant _time_zone => '';
 use constant _strptime  => '%Y-%m-%d %H:%M:%S';
@@ -25,8 +26,10 @@ minute, and second information is discarded when encoding.
 =head2 encode
 
 If value is DateTime object then converts it into ISO format
-C<YYYY-MM-DD hh:mm:ss>. Does nothing if value is not defined or
-string.
+C<YYYY-MM-DD hh:mm:ss>. Does nothing if value is not defined.
+
+Sets the value to undef if the value is a string and doesn't match an ISO date (at least).
+
 
 =cut
 
@@ -34,10 +37,17 @@ sub encode {
     my $self = shift;
 
     my $value_ref = $self->value_ref;
+
+    return if !defined $$value_ref;
+
+    if  ( ! UNIVERSAL::isa( $$value_ref, 'DateTime' )) {
+        if ( $$value_ref !~ /^\d{4}[ -]?\d{2}[ -]?[\d{2}]/) {
+       $$value_ref = undef;
+        }
+        return undef;
+   }
+
     return unless $$value_ref;
-
-    return unless UNIVERSAL::isa( $$value_ref, 'DateTime' );
-
     if (my $tz = $self->_time_zone) {
 	$$value_ref = $$value_ref->clone;
 	$$value_ref->time_zone('floating');
@@ -67,11 +77,17 @@ sub decode {
 # but we need Jifty::DBI::Handle here to get DB type
 
     my $str = join('T', split ' ', $$value_ref, 2);
-    my $dt = DateTime::Format::ISO8601->parse_datetime($str);
+    my $dt;
+    eval { $dt  = DateTime::Format::ISO8601->parse_datetime($str) };
+
+    if ($@) { # if datetime can't decode this, scream loudly with a useful error message
+        Carp::cluck($@);
+        return;
+    }
 
     if ($dt) {
 	my $tz = $self->_time_zone;
-	$dt->time_zone($tz) if $tz;
+	$dt->set_time_zone($tz) if $tz;
 
         $dt->set_formatter(DateTime::Format::Strptime->new(pattern => $self->_strptime));
         $$value_ref = $dt;
