@@ -61,9 +61,10 @@ sub new {
     return $self;
 }
 
-=head2 connect PARAMHASH: Driver, Database, Host, User, Password
+=head2 connect PARAMHASH
 
-Takes a paramhash and connects to your DBI datasource. 
+Takes a paramhash and connects to your DBI datasource, with the keys C<driver>,
+C<database>, C<host>, C<user> and C<password>.
 
 If you created the handle with 
      Jifty::DBI::Handle->new
@@ -134,6 +135,8 @@ sub _upgrade_handle {
 
     my $driver = shift;
     my $class  = 'Jifty::DBI::Handle::' . $driver;
+
+    local $@;
     eval "require $class";
     return if $@;
 
@@ -332,6 +335,27 @@ sub dbh {
     return ( $DBIHandle{$self} ||= $PrevHandle );
 }
 
+=head2 delete $table_NAME @KEY_VALUE_PAIRS
+
+Takes a table name and a set of key-value pairs in an array. splits the key value pairs, constructs an DELETE statement and performs the delete. Returns the row_id of this row.
+
+=cut
+
+sub delete {
+    my ( $self, $table, @pairs ) = @_;
+
+    my @bind  = ();
+    my $where = 'WHERE ';
+    while (my $key = shift @pairs) {
+        $where .= $key . "=?" . " AND ";
+        push( @bind, shift(@pairs) );
+    }
+
+    $where =~ s/AND $//;
+    my $query_string = "DELETE FROM " . $table . ' ' . $where;
+    $self->simple_query( $query_string, @bind );
+}
+
 =head2 insert $table_NAME @KEY_VALUE_PAIRS
 
 Takes a table name and a set of key-value pairs in an array. splits the key value pairs, constructs an INSERT statement and performs the insert. Returns the row_id of this row.
@@ -363,13 +387,13 @@ sub insert {
 
 =head2 update_record_value 
 
-Takes a hash with columns: Table, Column, Value PrimaryKeys, and 
-IsSQLFunction.  Table, and Column should be obvious, Value is where you 
-set the new value you want the column to have. The primary_keys column should 
-be the lvalue of Jifty::DBI::Record::PrimaryKeys().  Finally 
-IsSQLFunction is set when the Value is a SQL function.  For example, you 
-might have ('Value'=>'PASSWORD(string)'), by setting IsSQLFunction that 
-string will be inserted into the query directly rather then as a binding. 
+Takes a hash with columns: C<table>, C<column>, C<value>, C<primary_keys>, and
+C<is_sql_function>.  The first two should be obvious; C<value> is where you 
+set the new value you want the column to have. The C<primary_keys> column should 
+be the lvalue of Jifty::DBI::Record::PrimaryKeys().  Finally ,
+C<is_sql_function> is set when the Value is a SQL function.  For example, you 
+might have C<< value => 'PASSWORD(string)' >>, by setting C<is_sql_function> to true,
+that string will be inserted into the query directly rather then as a binding. 
 
 =cut
 
@@ -411,8 +435,9 @@ sub update_record_value {
 
 =head2 update_table_value table COLUMN NEW_value RECORD_ID IS_SQL
 
-Update column COLUMN of table table where the record id = RECORD_ID.  if IS_SQL is set,
-don\'t quote the NEW_VALUE
+Update column COLUMN of table table where the record id = RECORD_ID.
+
+If IS_SQL is set, don't quote the NEW_VALUE.
 
 =cut
 
@@ -480,6 +505,8 @@ sub simple_query {
         $basetime = Time::HiRes::time();
     }
     my $executed;
+
+    local $@;
     {
         no warnings 'uninitialized';    # undef in bind_values makes DBI sad
         eval { $executed = $sth->execute(@bind_values) };
@@ -634,8 +661,13 @@ sub _make_clause_case_insensitive {
     my $value    = shift;
 
     if ($self->_case_insensitivity_valid($column, $operator, $value)) {
-      $column = "lower($column)";
-      $value  = "lower($value)";
+        $column = "lower($column)";
+        if (ref $value eq 'ARRAY') {
+            map {$_ = "lower($_)"} @{$value};
+        }
+        else {
+            $value  = "lower($value)";
+        }
     }
     return ( $column, $operator, $value );
 }
@@ -726,13 +758,13 @@ sub force_rollback {
     $self->rollback(1);
 }
 
-=head2 transaction_depthh
+=head2 transaction_depth
 
 Return the current depth of the faked nested transaction stack.
 
 =cut
 
-sub transaction_depthh {
+sub transaction_depth {
     my $self = shift;
     return ($TRANSDEPTH);
 }
@@ -781,18 +813,16 @@ sub join {
     my %args = (
         collection => undef,
         type       => 'normal',
-        column1    => 'main',
-        alias1     => undef,
+        alias1     => 'main',
+        column1    => undef,
         table2     => undef,
-        column2    => undef,
         alias2     => undef,
+        column2    => undef,
         expression => undef,
         operator   => '=',
         @_
     );
 
-    my $string;
-
     my $alias;
 
     #If we're handed in an alias2, we need to go remove it from the
@@ -806,7 +836,7 @@ sub join {
         my @aliases = @{ $args{'collection'}->{'aliases'} };
         my @new_aliases;
         foreach my $old_alias (@aliases) {
-            if ( $old_alias =~ /^(.*?) ($args{'alias2'})$/ ) {
+            if ( $old_alias =~ /^(.*?) (\Q$args{'alias2'}\E)$/ ) {
                 $args{'table2'} = $1;
                 $alias = $2;
 
@@ -840,8 +870,8 @@ sub join {
 
         }
 
-        if ( !$alias || $args{'alias1'} ) {
-            return ( $self->_normal_join(%args) );
+        unless ( $alias ) {
+            return $self->_normal_join(%args);
         }
 
         $args{'collection'}->{'aliases'} = \@new_aliases;
@@ -852,32 +882,25 @@ sub join {
 
     }
 
+    my $meta = $args{'collection'}->{'leftjoins'}{ $alias } ||= {};
     if ( $args{'type'} =~ /LEFT/i ) {
-
-        $string = " LEFT JOIN " . $args{'table2'} . " $alias ";
+        $meta->{'alias_string'} = " LEFT JOIN " . $args{'table2'} . " $alias ";
+        $meta->{'type'} = 'LEFT';
 
     } else {
-
-        $string = " JOIN " . $args{'table2'} . " $alias ";
-
+        $meta->{'alias_string'} = " JOIN " . $args{'table2'} . " $alias ";
+        $meta->{'type'} = 'NORMAL';
     }
+    $meta->{'depends_on'} = $args{'alias1'};
+    $meta->{'entry_aggregator'} = $args{'entry_aggregator'}
+        if $args{'entry_aggregator'};
 
-    my $criterion;
-    if ( $args{'expression'} ) {
-        $criterion = $args{'expression'};
-    } else {
-        $criterion = $args{'alias1'} . "." . $args{'column1'};
-    }
-
-    $args{'collection'}->{'leftjoins'}{"$alias"}{'alias_string'} = $string;
-    $args{'collection'}->{'leftjoins'}{"$alias"}{'entry_aggregator'}
-        = $args{'entry_aggregator'}
-        if ( $args{'entry_aggregator'} );
-    $args{'collection'}->{'leftjoins'}{"$alias"}{'depends_on'}
-        = $args{'alias1'};
-    $args{'collection'}->{'leftjoins'}{"$alias"}{'criteria'}
-        { 'criterion' . $args{'collection'}->{'criteria_count'}++ }
-        = " $criterion $args{'operator'} $alias.$args{'column2'}";
+    my $criterion = $args{'expression'} || "$args{'alias1'}.$args{'column1'}";
+    $meta->{'criteria'}{ 'base_criterion' } = [{
+        column   => $criterion,
+        operator => $args{'operator'},
+        value    => "$alias.$args{'column2'}",
+    }];
 
     return ($alias);
 }
@@ -901,22 +924,25 @@ sub _normal_join {
 
     if ( $args{'type'} =~ /LEFT/i ) {
         my $alias = $sb->_get_alias( $args{'table2'} );
-
-        $sb->{'leftjoins'}{"$alias"}{'alias_string'}
-            = " LEFT JOIN $args{'table2'} $alias ";
-
-        $sb->{'leftjoins'}{"$alias"}{'criteria'}{'base_criterion'}
-            = " $args{'alias1'}.$args{'column1'} $args{'operator'} $alias.$args{'column2'}";
+        my $meta  = $sb->{'leftjoins'}{ $alias } ||= {};
+        $meta->{'alias_string'} = " LEFT JOIN $args{'table2'} $alias ";
+        $meta->{'depends_on'}   = $args{'alias1'};
+        $meta->{'type'}         = 'LEFT';
+        $meta->{'base_criterion'} = [ {
+            column   => "$args{'alias1'}.$args{'column1'}",
+            operator => $args{'operator'},
+            value    => "$alias.$args{'column2'}",
+        } ];
 
         return ($alias);
     } else {
         $sb->Jifty::DBI::Collection::limit(
             entry_aggregator => 'AND',
+            @_,
             quote_value      => 0,
             alias            => $args{'alias1'},
             column           => $args{'column1'},
             value            => $args{'alias2'} . "." . $args{'column2'},
-            @_
         );
     }
 }
@@ -927,48 +953,204 @@ sub _normal_join {
 sub _build_joins {
     my $self = shift;
     my $sb   = shift;
-    my %seen_aliases;
 
-    $seen_aliases{'main'} = 1;
+    $self->_optimize_joins( collection => $sb );
 
-    # We don't want to get tripped up on a dependency on a simple alias.
-    foreach my $alias ( @{ $sb->{'aliases'} } ) {
-        if ( $alias =~ /^(.*?)\s+(.*?)$/ ) {
-            $seen_aliases{$2} = 1;
+    my $join_clause = CORE::join " CROSS JOIN ", ($sb->table ." main"), @{ $sb->{'aliases'} };
+    my %processed = map { /^\S+\s+(\S+)$/; $1 => 1 } @{ $sb->{'aliases'} };
+    $processed{'main'} = 1;
+
+    # get a @list of joins that have not been processed yet, but depend on processed join
+    my $joins = $sb->{'leftjoins'};
+    while ( my @list = grep !$processed{ $_ }
+            && $processed{ $joins->{ $_ }{'depends_on'} }, keys %$joins )
+    {
+        foreach my $join ( @list ) {
+            $processed{ $join }++;
+
+            my $meta = $joins->{ $join };
+            my $aggregator = $meta->{'entry_aggregator'} || 'AND';
+
+            $join_clause .= $meta->{'alias_string'} . " ON ";
+            my @tmp = map {
+                    ref($_)?
+                        $_->{'column'} .' '. $_->{'operator'} .' '. $_->{'value'}:
+                        $_
+                }
+                map { ('(', @$_, ')', $aggregator) } values %{ $meta->{'criteria'} };
+            # delete last aggregator
+            pop @tmp;
+            $join_clause .= CORE::join ' ', @tmp;
         }
     }
 
-    my $join_clause = $sb->table . " main ";
+    # here we could check if there is recursion in joins by checking that all joins
+    # are processed
+    if ( my @not_processed = grep !$processed{ $_ }, keys %$joins ) {
+        die "Unsatisfied dependency chain in joins @not_processed";
+    }
+    return $join_clause;
+}
 
-    my @keys = ( keys %{ $sb->{'leftjoins'} } );
-    my %seen;
+sub _optimize_joins {
+    my $self = shift;
+    my %args = ( collection => undef, @_ );
+    my $joins = $args{'collection'}->{'leftjoins'};
 
-    while ( my $join = shift @keys ) {
-        if ( !$sb->{'leftjoins'}{$join}{'depends_on'}
-            || $seen_aliases{ $sb->{'leftjoins'}{$join}{'depends_on'} } )
-        {
-            $join_clause
-                .= $sb->{'leftjoins'}{$join}{'alias_string'} . " ON ";
+    my %processed = map { /^\S+\s+(\S+)$/; $1 => 1 } @{ $args{'collection'}->{'aliases'} };
+    $processed{ $_ }++ foreach grep $joins->{ $_ }{'type'} ne 'LEFT', keys %$joins;
+    $processed{'main'}++;
 
-            my @criteria = values %{ $sb->{'leftjoins'}{$join}{'criteria'} };
-            my $entry_aggregator
-                = $sb->{'leftjoins'}{$join}{'entry_aggregator'} || 'AND';
-            my $criteria = CORE::join( " $entry_aggregator ",
-                map {" ( $_ ) "} @criteria );
+    my @ordered;
+    # get a @list of joins that have not been processed yet, but depend on processed join
+    # if we are talking about forest then we'll get the second level of the forest,
+    # but we should process nodes on this level at the end, so we build FILO ordered list.
+    # finally we'll get ordered list with leafes in the beginning and top most nodes at
+    # the end.
+    while ( my @list = grep !$processed{ $_ }
+            && $processed{ $joins->{ $_ }{'depends_on'} }, keys %$joins )
+    {
+        unshift @ordered, @list;
+        $processed{ $_ }++ foreach @list;
+    }
 
-            $join_clause .= "( " . $criteria . " ) ";
-            $join_clause = "(" . $join_clause . ")";
+    foreach my $join ( @ordered ) {
+        next if $self->may_be_null( collection => $args{'collection'}, alias => $join );
 
-            $seen_aliases{$join} = 1;
+        $joins->{ $join }{'alias_string'} =~ s/^\s*LEFT\s+/ /i;
+        $joins->{ $join }{'type'} = 'NORMAL';
+    }
+
+}
+
+=head2 may_be_null
+
+Takes a C<collection> and C<alias> in a hash and returns
+true if restrictions of the query allow NULLs in a table joined with
+the alias, otherwise returns false value which means that you can
+use normal join instead of left for the aliased table.
+
+Works only for queries have been built with L<Jifty::DBI::Collection/join> and
+L<Jifty::DBI::Collection/limit> methods, for other cases return true value to
+avoid fault optimizations.
+
+=cut
+
+sub may_be_null {
+    my $self = shift;
+    my %args = (collection => undef, alias => undef, @_);
+    # if we have at least one subclause that is not generic then we should get out
+    # of here as we can't parse subclauses
+    return 1 if grep $_ ne 'generic_restrictions', keys %{ $args{'collection'}->{'subclauses'} };
+
+    # build full list of generic conditions
+    my @conditions;
+    foreach ( grep @$_, values %{ $args{'collection'}->{'restrictions'} } ) {
+        push @conditions, 'AND' if @conditions;
+        push @conditions, '(', @$_, ')';
+    }
+
+    # find tables that depends on this alias and add their join conditions
+    foreach my $join ( values %{ $args{'collection'}->{'leftjoins'} } ) {
+        # left joins on the left side so later we'll get 1 AND x expression
+        # which equal to x, so we just skip it
+        next if $join->{'type'} eq 'LEFT';
+        next unless $join->{'depends_on'} eq $args{'alias'};
+
+        my @tmp = map { ('(', @$_, ')', $join->{'entry_aggregator'}) } values %{ $join->{'criteria'} };
+        pop @tmp;
+
+        @conditions = ('(', @conditions, ')', 'AND', '(', @tmp ,')');
+
+    }
+    return 1 unless @conditions;
+
+    # replace conditions with boolean result: 1 - allow nulls, 0 - doesn't
+    foreach ( splice @conditions ) {
+        unless ( ref $_ ) {
+            push @conditions, $_;
+        } elsif ( $_->{'column'} =~ /^\Q$args{'alias'}./ ) {
+            # only operator IS allows NULLs in the aliased table
+            push @conditions, lc $_->{'operator'} eq 'is';
+        } elsif ( $_->{'value'} && $_->{'value'} =~ /^\Q$args{'alias'}./ ) {
+            # right operand is our alias, such condition don't allow NULLs
+            push @conditions, 0;
         } else {
-            push( @keys, $join );
-            die "Unsatisfied dependency chain in joins @keys"
-                if $seen{"@keys"}++;
+            # conditions on other aliases
+            push @conditions, 1;
         }
-
     }
-    return ( CORE::join( ", ", ( $join_clause, @{ $sb->{'aliases'} } ) ) );
 
+    # returns index of closing paren by index of openning paren
+    my $closing_paren = sub {
+        my $i = shift;
+        my $count = 0;
+        for ( ; $i < @conditions; $i++ ) {
+            if ( $conditions[$i] eq '(' ) {
+                $count++;
+            }
+            elsif ( $conditions[$i] eq ')' ) {
+                $count--;
+            }
+            return $i unless $count;
+        }
+        die "lost in parens";
+    };
+
+    # solve boolean expression we have, an answer is our result
+    my @tmp = ();
+    while ( defined ( my $e = shift @conditions ) ) {
+        #warn "@tmp >>>$e<<< @conditions";
+        return $e if !@conditions && !@tmp;
+
+        unless ( $e ) {
+            if ( $conditions[0] eq ')' ) {
+                push @tmp, $e;
+                next;
+            }
+
+            my $aggreg = uc shift @conditions;
+            if ( $aggreg eq 'OR' ) {
+                # 0 OR x == x
+                next;
+            } elsif ( $aggreg eq 'AND' ) {
+                # 0 AND x == 0
+                my $close_p = $closing_paren->(0);
+                splice @conditions, 0, $close_p + 1, (0);
+            } else {
+                die "lost @tmp >>>$e $aggreg<<< @conditions";
+            }
+        } elsif ( $e eq '1' ) {
+            if ( $conditions[0] eq ')' ) {
+                push @tmp, $e;
+                next;
+            }
+
+            my $aggreg = uc shift @conditions;
+            if ( $aggreg eq 'OR' ) {
+                # 1 OR x == 1
+                my $close_p = $closing_paren->(0);
+                splice @conditions, 0, $close_p + 1, (1);
+            } elsif ( $aggreg eq 'AND' ) {
+                # 1 AND x == x
+                next;
+            } else {
+                die "lost @tmp >>>$e $aggreg<<< @conditions";
+            }
+        } elsif ( $e eq '(' ) {
+            if ( $conditions[1] eq ')' ) {
+                splice @conditions, 1, 1;
+            } else {
+                push @tmp, $e;
+            }
+        } elsif ( $e eq ')' ) {
+            unshift @conditions, @tmp, $e;
+            @tmp = ();
+        } else {
+            die "lost: @tmp >>>$e<<< @conditions";
+        }
+    }
+    return 1;
 }
 
 =head2 distinct_query STATEMENTREF 
@@ -1006,7 +1188,7 @@ sub distinct_count {
 
 }
 
-=head2 Log MESSAGE
+=head2 log MESSAGE
 
 Takes a single argument, a message to log.
 
