@@ -38,16 +38,143 @@ other attributes for database structure that are not exposed directly
 here.  One example of this is the "refers_to" method used to create
 associations between classes.
 
+It re-exports C<defer> and C<lazy> from L<Scalar::Defer>, for setting
+parameter fields that must be recomputed at request-time:
+
+    column name =>
+        default is defer { Jifty->web->current_user->name };
+
+See L<Scalar::Defer> for more information about C<defer>.
+
 =cut
 
 use Carp qw/croak carp/;
-use Exporter::Lite;
-our @EXPORT
-    = qw(column type default literal validator autocompleted immutable unreadable max_length length distinct mandatory not_null sort_order valid_values label hints render_as render since till input_filters output_filters filters virtual is as by are on schema indexed valid order);
+use Scalar::Defer;
+use base qw(Class::Data::Inheritable);
+__PACKAGE__->mk_classdata('TYPES' => {});
+
+use Object::Declare (
+    mapping => {
+        column => sub { Jifty::DBI::Column->new({@_}) } ,
+    },
+    aliases => {
+        default_value => 'default',
+        available   => 'available_values',
+        valid       => 'valid_values',
+        render      => 'render_as',
+        order       => 'sort_order',
+        filters     => 'input_filters',
+    },
+    copula  => {
+        is          => sub { return @_ if $#_;
+                             my $typehandler = __PACKAGE__->TYPES->{$_[0]};
+                             # XXX: when we have a type name
+                             # convention, give a warning when it
+                             # looks like a type name but not found
+                             return ($_[0] => 1) unless $typehandler;
+                             return $typehandler->();
+			 },
+        are         => '',
+        as          => '',
+        ajax        => 'ajax_',
+        refers_to   => sub { refers_to => @_ },
+        references  => sub { refers_to => @_ },
+    },
+);
+use Class::Data::Inheritable;
+use UNIVERSAL::require ();
+
+our @EXPORT = qw( defer lazy column schema by render_as since till literal);
+
+sub by ($) { @_ }
+sub render_as ($) { render as @_ }
+sub since ($) { since is @_ }
+sub till ($) { till is @_ }
+
+sub literal($) {
+    my $value = shift;
+    return \$value;
+}
 
 our $SCHEMA;
 our $SORT_ORDERS = {};
 
+use Exporter::Lite ();
+# TODO - This "sub import" is strictly here to catch the deprecated "length is 40".
+#        Once the deprecation cycle is over we should take this away and rever to
+#        "use Exporter::Lite" in the line above.
+my $old_sig_die;
+
+sub import {
+    no warnings qw( uninitialized numeric );
+    $old_sig_die ||= $SIG{__DIE__};
+    $SIG{__DIE__} = \&filter_die unless $SIG{__DIE__} and $SIG{__DIE__} == \&filter_die;
+    goto &Exporter::Lite::import;
+}
+
+=head2 filter_die
+
+=cut
+
+sub filter_die {
+    # Calling it by hand means we restore the old sighandler.
+    $SIG{__DIE__} = $old_sig_die;
+    if ($_[0] =~ /near "is (\d+)"/) {
+        carp @_, << ".";
+
+*********************************************************
+
+ Due to an incompatible API change, the "length" field in
+ Jifty::DBI columns has been renamed to "max_length":
+ 
+     column foo =>
+         length is $1;       # NOT VALID 
+
+ Please write this instead:
+ 
+     column foo =>
+         max_length is $1    # VALID
+
+ Sorry for the inconvenience.
+
+**********************************************************
+
+
+.
+        exit 1;
+    }
+    elsif ($_[0] =~ /Undefined subroutine &Jifty::DBI::Schema::column|Can't locate object method "type" via package "(?:is|are)"/) {
+        my $from = (caller)[0];
+        $from =~ s/::Schema$//;
+        my $base = $INC{'Jifty/Record.pm'} ? "Jifty::Record" : "Jifty::DBI::Record";
+
+        no strict 'refs';
+        carp @_, << ".";
+*********************************************************
+
+ Calling 'column' within a schema class is an error:
+ 
+    package $from\::Schema;
+    column foo => ...;        # NOT VALID
+
+ Please write this instead:
+
+    package $from;
+    use Jifty::DBI::Schema;
+    use @{[(${"$from\::ISA"} || [$base])->[0] || $base]} schema {
+        column foo => ...;    # VALID
+    };
+
+ Sorry for the inconvenience.
+
+*********************************************************
+.
+    }
+
+    die @_;
+}
+
+
 =head1 FUNCTIONS
 
 All these functions are exported.  However, if you use the C<schema> helper function,
@@ -66,43 +193,91 @@ If your application subclasses C<::Record>, then write this instead:
 
 =cut
 
-sub schema (&) {
-    my $code = shift;
-
-    my $from = (caller)[0];
-
-    my $new_code = sub {
-        $code->();
-
-        # Unimport all our symbols from the calling package.
-        foreach my $sym (@EXPORT) {
-            no strict 'refs';
-            undef *{"$from\::$sym"}
-                if \&{"$from\::$sym"} == \&$sym;
-        }
-
-        # Then initialize all columns
-        foreach my $column (sort keys %{$from->COLUMNS||{}}) {
-            $from->_init_methods_for_column($from->COLUMNS->{$column});
-        }
-    };
-
-    return('-base' => $new_code);
-}
-
 =head2 column
 
-Set forth the description of a column in the data store.
-
-Note: If the column uses 'refers_to' to reference another class then you 
-should not name the column ending in '_id' as it has special meaning.
+DEPRECATED.  This method of defining columns will not work anymore.  Please
+use the C<schema {}> method documented above.
 
 =cut
 
-sub column {
-    my $name = lc(shift);
+sub schema (&) {
+    my $code = shift;
+    my $from = caller;
 
-    my $from = (caller)[0];
+    my $new_code = sub {
+	no warnings 'redefine';
+	local *_ = sub { my $args = \@_; defer { _(@$args) } };
+	$from->_init_columns;
+
+	my @columns = &declare($code);
+
+	# Unimport all our symbols from the calling package,
+        # except for "lazy" and "defer".
+	foreach my $sym (@EXPORT) {
+            next if $sym eq 'lazy' or $sym eq 'defer';
+
+	    no strict 'refs';
+	    undef *{"$from\::$sym"}
+		if \&{"$from\::$sym"} == \&$sym;
+	}
+
+	foreach my $column (@columns) {
+	    next if !ref($column);
+	    _init_column($column);
+	}
+
+        $from->_init_methods_for_columns;
+    };
+
+    return ('-base' => $new_code);
+}
+
+use Hash::Merge ();
+
+no warnings 'uninitialized';
+use constant MERGE_PARAM_BEHAVIOUR => {
+    SCALAR => {
+            SCALAR => sub { CORE::length($_[1]) ? $_[1] : $_[0] },
+            ARRAY  => sub { [ @{$_[1]} ] },
+            HASH   => sub { $_[1] } },
+    ARRAY => {
+            SCALAR => sub { CORE::length($_[1]) ? $_[1] : $_[0] },
+            ARRAY  => sub { [ @{$_[1]} ] },
+            HASH   => sub { $_[1] } },
+    HASH => {
+            SCALAR => sub { CORE::length($_[1]) ? $_[1] : $_[0] },
+            ARRAY  => sub { [ @{$_[1]} ] },
+            HASH   => sub { Hash::Merge::_merge_hashes( $_[0], $_[1] ) } }
+};
+
+=head2 merge_params HASHREF HASHREF
+
+Takes two hashrefs. Merges them together and returns the merged hashref.
+
+    - Empty fields in subclasses don't override nonempty fields in superclass anymore.
+    - Arrays don't merge; e.g. if parent class's valid_values is [1,2,3,4], and
+      subclass's valid_values() is [1,2], they don't somehow become [1,2,3,4,1,2].
+
+BUG: This should either be a private routine or factored out into Jifty::Util
+
+
+
+=cut
+
+sub merge_params {
+    my $prev_behaviour = Hash::Merge::get_behavior();
+    Hash::Merge::specify_behavior( MERGE_PARAM_BEHAVIOUR, "merge_params" );
+    my $rv = Hash::Merge::merge(@_);
+    Hash::Merge::set_behavior( $prev_behaviour );
+    return $rv;
+}
+
+
+sub _init_column {
+    my $column = shift;
+    my $name   = $column->name;
+
+    my $from = (caller(2))[0];
     if ($from =~ s/::Schema$// && $from !~ /Script/) {
         no strict 'refs';
 
@@ -128,6 +303,13 @@ sub column {
 *********************************************************
 .
     }
+    return _init_column_for($column, $from, @_);
+}
+
+sub _init_column_for {
+    my $column = shift;
+    my $from   = shift;
+    my $name   = $column->name;
 
     croak "Base of schema class $from is not a Jifty::DBI::Record"
       unless UNIVERSAL::isa($from, "Jifty::DBI::Record");
@@ -135,55 +317,68 @@ sub column {
     croak "Illegal column definition for column $name in $from"
       if grep {not UNIVERSAL::isa($_, "Jifty::DBI::Schema::Trait")} @_;
 
-    $from->_init_columns;
+    $column->readable(!(delete $column->{unreadable}));
+    $column->writable(!(delete $column->{immutable}));
 
+    # XXX: deprecated
+    $column->mandatory(1) if delete $column->{not_null};
 
-    my @args = (
-        ! unreadable(),
-        ! immutable(),
-        ! virtual(),
-        type(''),
-        @_
-    );
-
-    my $column = Jifty::DBI::Column->new( { name => $name } );
     $column->sort_order($SORT_ORDERS->{$from}++);
 
-    $_->apply($column) for @args;
+    $column->input_filters($column->{input_filters} || []);
+    $column->output_filters($column->{output_filters} || []);
 
+    # Set up relationships to other records and collections
     if ( my $refclass = $column->refers_to ) {
+        
+        # Handle refers_to SomeCollection by 'foo'
+        if (ref($refclass) eq 'ARRAY') {
+            $column->by($refclass->[1]);
+            $column->refers_to($refclass = $refclass->[0]);
+        }
+
+        # Load the class we reference
         $refclass->require();
+
+        # References assume a refernce to an integer ID unless told otherwise
         $column->type('integer') unless ( $column->type );
 
+        # A one-to-one or one-to-many relationship is requested
         if ( UNIVERSAL::isa( $refclass, 'Jifty::DBI::Record' ) ) {
+
+            # Handle *_id reference columns specially
             if ( $name =~ /(.*)_id$/ ) {
                 my $aliased_as = $1;
                 my $virtual_column = $from->add_column($aliased_as);
 
-                # XXX FIXME I think the next line is wrong, but things
-                # explode without it -- mostly because we unique-key
-                # on name instead of some conbination of name and
-                # alias_for_column in a couple places
-                $virtual_column->name( $name );
-                $virtual_column->aliased_as($aliased_as);
-                $_->apply($virtual_column) for @args;
+                # Clone ourselves into the virtual column
+                %$virtual_column = %$column;
+
+                # This column is now just the ID, the virtual holds the ref
                 $column->refers_to(undef);
+
+                # Note the original column
+                $virtual_column->aliased_as($aliased_as);
                 $virtual_column->alias_for_column($name);
+
+                # Create the helper methods for the virtual column too
                 $from->_init_methods_for_column($virtual_column);
             }
+
+            # Assume we refer to the ID column unless told otherwise
             $column->by('id') unless $column->by;
-            $column->type('integer') unless $column->type;
         } elsif ( UNIVERSAL::isa( $refclass, 'Jifty::DBI::Collection' ) ) {
             $column->by('id') unless $column->by;
             $column->virtual('1');
         } else {
-            warn "Error: $refclass neither Record nor Collection";
+            warn "Error in $from: $refclass neither Record nor Collection";
         }
+    } elsif (my $handler = $column->{_init_handler}) {
+        $handler->($column, $from);
     } else {
         $column->type('varchar(255)') unless $column->type;
     }
 
-
     $from->COLUMNS->{$name} = $column;
 
     # Heuristics: If we are called through Jifty::DBI::Schema, 
@@ -191,19 +386,30 @@ sub column {
     # through the &schema wrapper, so we defer initialization here
     # to not upset column names such as "label" and "type".
     # (We may not *have* a caller(1) if the user is executing a .pm file.)
-    my $caller1 = caller(1);
-    return if defined $caller1 && $caller1 eq __PACKAGE__;
-
-    $from->_init_methods_for_column($column)
 }
 
-=head2 refers_to
+=head2 register_types
+
+=cut
+
+sub register_types {
+    my $class = shift;
+    while (my ($type, $sub) = splice(@_, 0, 2)) {
+        $class->TYPES->{$type} = $sub;
+    }
+}
+
+1;
+
+__END__
+
+=head2 references
 
 Indicates that the column references an object or a collection of objects in another
 class.  You may refer to either a class that inherits from Jifty::Record by a primary
 key in that class or to a class that inherits from Jifty::Collection.
 
-Correct usage is C<refers_to Application::Model::OtherClass by 'column_name'>, where
+Correct usage is C<references Application::Model::OtherClass by 'column_name'>, where
 Application::Model::OtherClass is a valid Jifty model and C<'column_name'> is
 a column containing unique values in OtherClass.  You can omit C<by 'column_name'> and
 the column name 'id' will be used.
@@ -216,10 +422,19 @@ columns referring to Jifty::Collections.
 
 For columns referring to Jifty::Records you can access the actual value of the column
 instead of the object reference by appending '_id' to the column name.  As a result,
-you may not end any column name which uses 'refers_to' using '_id'.
+you may not end any column name which uses 'references' using '_id'.
+
+=head2 refers_to
+
+Synonym for C<references>.
 
 =cut
 
+=head2 by
+
+Helper for C<references>.  Used to specify what column name should be
+used in the referenced model.  See the documentation for C<references>e
+
 =head2 type
 
 type passed to our database abstraction layer, which should resolve it
@@ -233,59 +448,28 @@ The impact of this is that not all column types are portable between
 databases.  For example blobs have different names between
 mysql and postgres.
 
-=cut
-
-sub type {
-    _list( type => @_ );
-}
-
 =head2 default
 
 Give a default value for the column.  Correct usage is C<default is
 'foo'>.
 
-=cut
-
-sub default {
-    _list( default => @_ );
-}
-
 =head2 literal
 
 Used for default values, to connote that they should not be quoted
 before being supplied as the default value for the column.  Correct
 usage is C<default is literal 'now()'>.
 
-=cut
-
-sub literal($) {
-    my $value = shift;
-    return \$value;
-}
-
 =head2 validator
 
 Defines a subroutine which returns a true value only for valid values
 this column can have.  Correct usage is C<validator is \&foo>.
 
-=cut
-
-sub validator {
-    _list( validator => @_ );
-}
-
 =head2 immutable
 
 States that this column is not writable.  This is useful for
 properties that are set at creation time but not modifiable
 thereafter, like 'created by'.  Correct usage is C<is immutable>.
 
-=cut
-
-sub immutable {
-    _item( writable => 0, @_ );
-}
-
 =head2 unreadable
 
 States that this column is not directly readable by the application
@@ -293,93 +477,29 @@ using C<< $record->column >>; this is useful for password columns and
 the like.  The data is still accessible via C<< $record->_value('') >>.
 Correct usage is C<is unreadable>.
 
-=cut
-
-sub unreadable {
-    _item( readable => 0, @_ );
-}
-
 =head2 max_length
 
-Sets a maximum length to store in the database; values longer than
+Sets a maximum max_length to store in the database; values longer than
 this are truncated before being inserted into the database, using
 L<Jifty::DBI::Filter::Truncate>.  Note that this is in B<bytes>, not
 B<characters>.  Correct usage is C<max_length is 42>.
 
-=cut
-
-sub max_length {
-    _list( max_length => @_ );
-}
-
-=head2 length
-
-DEPRECATED alias for C<max_length>.  Do not use; this will go away in the
-next version.
-
-=cut
-
-sub length {
-    no strict 'refs';
-    carp << "." unless caller()->{_seen_length_warning}++;
-
-*********************************************************
-
- Due to an incompatible API change, the "length" field in
- Jifty::DBI columns has been renamed to "max_length":
- 
-     column foo =>
-         length is $_[0];       # NOT VALID 
-
- Please write this instead:
- 
-     column foo =>
-         max_length is $_[0]    # VALID
-
- Sorry for the inconvenience.
-
-**********************************************************
-
-
-.
-    _list( max_length => @_ );
-}
-
 
 =head2 mandatory
 
 Mark as a required column.  May be used for generating user
 interfaces.  Correct usage is C<is mandatory>.
 
-=cut
-
-sub mandatory {
-    _item( mandatory => 1, @_ );
-}
-
 =head2 not_null
 
 Same as L</mandatory>.  This is deprecated.  Currect usage would be
 C<is not_null>.
 
-=cut
-
-sub not_null {
-    carp "'is not_null' is deprecated in favor of 'is mandatory'";
-    _item( mandatory => 1, @_ );
-}
-
 =head2 autocompleted
 
 Mark as an autocompleted column.  May be used for generating user
 interfaces.  Correct usage is C<is autocompleted>.
 
-=cut
-
-sub autocompleted {
-    _item( autocompleted => 1, @_ );
-}
-
 =head2 distinct
 
 Declares that a column should only have distinct values.  This
@@ -389,56 +509,26 @@ themselves. This is because there is no support for distinct
 columns implemented in L<DBIx::DBSchema> at this time.  
 Correct usage is C<is distinct>.
 
-=cut
-
-sub distinct {
-    _item( distinct => 1, @_ );
-}
-
 =head2 virtual
 
 Declares that a column is not backed by an actual column in the
 database, but is instead computed on-the-fly.
 
-=cut
-
-sub virtual {
-    _item( virtual => 1, @_ );
-}
-
-
 =head2 sort_order
 
 Declares an integer sort value for this column. By default, Jifty will sort
 columns in the order they are defined.
 
-=cut
-
-sub sort_order {
-    _item ( sort_order => (shift @_ || 0));
-}
-
 =head2 order
 
 Alias for C<sort_order>.
 
-=cut
-
-sub order { sort_order(@_) }
-
-
 =head2 input_filters
 
 Sets a list of input filters on the data.  Correct usage is
 C<input_filters are 'Jifty::DBI::Filter::DateTime'>.  See
 L<Jifty::DBI::Filter>.
 
-=cut
-
-sub input_filters {
-    _list( input_filters => @_ );
-}
-
 =head2 output_filters
 
 Sets a list of output filters on the data.  Correct usage is
@@ -446,12 +536,6 @@ C<output_filters are 'Jifty::DBI::Filter::DateTime'>.  See
 L<Jifty::DBI::Filter>.  You usually don't need to set this, as the
 output filters default to the input filters in reverse order.
 
-=cut
-
-sub output_filters {
-    _list( output_filters => @_ );
-}
-
 =head2 filters
 
 Sets a list of filters on the data.  These are applied when reading
@@ -460,27 +544,17 @@ B<and> writing to the database.  Correct usage is C<filters are
 actuality, this is the exact same as L</input_filters>, since output
 filters default to the input filters, reversed.
 
-=cut
-
-sub filters {
-    _list( input_filters => @_ );
-}
-
 =head2 since
 
 What application version this column was last changed.  Correct usage
 is C<since '0.1.5'>.
 
-=cut
-
-sub since {
-    _list( since => @_ );
-}
-
 =head2 till
 
-What application version this column was last supported.  Correct usage
-is C<till '0.2.5'>.
+The version after this column was supported. The column is not available in
+the version named, but would have been in the version immediately prior.
+
+Correct usage is C<till '0.2.5'>. This indicates that the column is not available in version C<0.2.5>, but was available in C<0.2.4>. The value specified for L</since> must be less than this version.
 
 =cut
 
@@ -503,45 +577,21 @@ and value.
   { display => 'Blue', value => 'blue' },
   { display => 'Red', value => 'red' }
 
-=cut
-
-sub valid_values {
-    _list( valid_values => @_ );
-}
-
 =head2 valid
 
 Alias for C<valid_values>.
 
-=cut
-
-sub valid {
-    _list( valid_values => @_ );
-}
-
 =head2 label
 
 Designates a human-readable label for the column, for use in user
 interfaces.  Correct usage is C<label is 'Your foo value'>.
 
-=cut
-
-sub label {
-    _list( label => @_ );
-}
-
 =head2 hints
 
 A sentence or two to display in long-form user interfaces about what
 might go in this column.  Correct usage is C<hints is 'Used by the
 frobnicator to do strange things'>.
 
-=cut
-
-sub hints {
-    _list( hints => @_ );
-}
-
 =head2 render_as
 
 Used in user interface generation to know how to render the column.
@@ -585,108 +635,15 @@ internal fields that should not actually be displayed.
 If these don't meet your needs, you can write your own subclass of
 L<Jifty::Web::Form::Field>. See the documentation for that module.
 
-=cut
-
-sub render_as {
-    _list( render_as => @_ );
-}
-
 =head2 render
 
 Alias for C<render_as>.
 
-=cut
-
-sub render {
-    _list( render_as => @_ );
-}
-
 =head2 indexed
 
 An index will be built on this column
 Correct usage is C<is indexed>
 
-=cut
-
-sub indexed {
-    _list( indexed => 1, @_ );
-}
-
-=head2 by
-
-Helper method to improve readability.
-
-=cut
-
-sub by {
-    _list( by => @_ );
-}
-
-=head2 is
-
-Helper method to improve readability.
-
-=cut
-
-sub is {
-    my $thing = shift;
-    ref $thing eq "ARRAY" ? ( @{$thing}, @_ ) : ($thing, @_);
-}
-
-=head2 as
-
-Helper method to improve readability.
-
-=cut
-
-sub as {
-    my $thing = shift;
-    ref $thing eq "ARRAY" ? ( @{$thing}, @_ ) : ($thing, @_);
-}
-
-=head2 are
-
-Helper method to improve readability.
-
-=cut
-
-sub are {
-    my $ref = [];
-    push @{$ref}, shift @_ while @_ and not UNIVERSAL::isa($_[0], "Jifty::DBI::Schema::Trait");
-    return( $ref, @_ );
-}
-
-=head2 on
-
-Helper method to improve readability.
-
-=cut
-
-sub on {
-    _list( self => shift );
-}
-
-sub _list {
-    defined wantarray
-        or croak("Cannot add traits in void context -- check for misspelled preceding comma as a semicolon or missing use statements for models you refer_to.");
-
-    wantarray
-        or croak("Cannot call list traits in scalar context -- check for unneccessary 'is'");
-    _trait(@_);
-}
-
-sub _item {
-    defined wantarray
-        or croak("Cannot add traits in void context -- check for misspelled preceding comma as a semicolon");
-
-    _trait(@_);
-}
-
-sub _trait {
-    my @trait;
-    push @trait, shift @_ while @_ and not UNIVERSAL::isa($_[0], "Jifty::DBI::Schema::Trait");
-    return wantarray ? (Jifty::DBI::Schema::Trait->new(@trait), @_) : Jifty::DBI::Schema::Trait->new(@trait);
-}
 
 =head1 EXAMPLE
 
@@ -703,32 +660,4 @@ under the same terms as Perl itself.
 
 =cut
 
-package Jifty::DBI::Schema::Trait;
-
-use overload "!" => \&negation;
-use Carp qw/croak/;
-
-sub new {
-    my $class = shift;
-    return bless [@_], $class;
-}
-
-sub apply {
-    my $self = shift;
-    my ($column) = @_;
-
-    my ($method, $argument) = @{$self};
-
-    croak("Illegal Jifty::DBI::Schema property '$method'")
-      unless $column->can($method);
-
-    $column->$method($argument);
-}
-
-sub negation {
-    my $self = shift;
-    my ($trait, @rest) = @{$self};
-    return (ref $self)->new($trait, map {not $_} @rest);
-}
-
 1;
